/*
* ausearch-llist.c - Minimal linked list library
* Copyright (c) 2005 Red Hat Inc., Durham, North Carolina.
* All Rights Reserved. 
*
* This software may be freely redistributed and/or modified under the
* terms of the GNU General Public License as published by the Free
* Software Foundation; either version 2, or (at your option) any
* later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; see the file COPYING. If not, write to the
* Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
* Authors:
*   Steve Grubb <sgrubb@redhat.com>
*/

#include <stdlib.h>
#include <string.h>
#include "ausearch-string.h"

void slist_create(slist *l)
{
	l->head = 0;
	l->cur = 0;
	l->cnt = 0;
}

void slist_first(slist *l)
{
	l->cur = l->head;
}

void slist_last(slist *l)
{
        register snode* window;
	
	if (l->head == NULL)
		return;

        window = l->head;
	while (window->next)
		window = window->next;
	l->cur = window;
}

snode *slist_next(slist *l)
{
	if (l->cur == NULL)
		return NULL;
	l->cur = l->cur->next;
	return l->cur;
}

snode *slist_prev(slist *l)
{
	if (l->cur == NULL)
		return NULL;

	if (l->cur->item <= 0)
		return NULL;

	slist_find_item(l, l->cur->item-1);
	return l->cur;
}

snode *slist_get_cur(slist *l)
{
	return l->cur;
}

void slist_append(slist *l, snode *node)
{
	snode* newnode;

	newnode = malloc(sizeof(snode));

	if (node->str)
		newnode->str = node->str;
	else
		newnode->str = 0;

	newnode->item = l->cnt; 
	newnode->next = 0;

	// if we are at top, fix this up
	if (l->head == 0)
		l->head = newnode;
	else	// Otherwise add pointer to newnode
		l->cur->next = newnode;

	// make newnode current
	l->cur = newnode;
	l->cnt++;
}

int slist_find_item(slist *l, unsigned int i)
{
        register snode* window;
                                                                                
	if (l->cur && (l->cur->item <= i))
		window = l->cur;	/* Try to use where we are */
	else
        	window = l->head;	/* Can't, start over */

	while (window)
	{
		if (window->item == i)
		{
			l->cur = window;
			return 1;
		}
		else
			window = window->next;
	}
	return 0;
}

void slist_clear(slist* l)
{
	snode* nextnode;
	register snode* current;

	current = l->head;
	while (current)
	{
		nextnode=current->next;
		if (current->str)
			free(current->str);
		free(current);
		current=nextnode;
	}
	l->head = 0;
	l->cur = 0;
	l->cnt = 0;
}

